/*
 
  copyright (c) 2004, Michal Salaban <emes@pld-linux.org>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License, version 2 as
  published by the Free Software Foundation (see file COPYING for details).

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
  $Id: conditions.c,v 1.2 2004/02/08 15:32:15 emes Exp $
 
*/
#include <stdio.h>
#include "parseopt/lex.h"
#include "conditions.h"

int ordcmp (int le, int eq, int gt, token op)
{
    switch (op) {
        case CO_LT:
            return le;
        case CO_LE:
            return (le || eq);
        case CO_GT:
            return gt;
        case CO_GE:
            return (gt || eq);
        case CO_EQ:
            return eq;
        case CO_NEQ:
            return ! eq;
        default:
            return 0;
    }
}

int logcmp (int a, token op, int b)
{
    switch (op) {
        case LO_OR:
            return (a || b);
        case LO_AND:
            return (a && b);
        default:
            return 0;
    }
}

int check_atomic (atomic *at)
{
    double      *dv = (double *) at->val,
                *dt = (double *) at->thresh;
    long int    *iv = (long int *) at->val,
                *it = (long int *) at->thresh;
    
    switch (at->type) {
        case DOUBLE:
            return ordcmp (
                    (*dv < *dt),
                    (*dv == *dt),
                    (*dv > *dt),
                    at->op
                    );
        case SIZE:
            return ordcmp (
                    (*iv < *it),
                    (*iv == *it),
                    (*iv > *it),
                    at->op
                    );
        default:
            return 0;
    }
}

int check_expression (expression *ex)
{
    switch (ex->type) {
        case ATOMIC:
            return check_atomic ((atomic *) ex->arg1);
        case RELATION:
            return logcmp (
                    check_expression ((expression *) ex->arg1),
                    ex->op,
                    check_expression ((expression *) ex->arg2)
                    );
        default:
            return 0;
    }
}

/* $Id: conditions.c,v 1.2 2004/02/08 15:32:15 emes Exp $ */
