/*
 
  copyright (c) 2004, Michal Salaban <emes@pld-linux.org>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License, version 2 as
  published by the Free Software Foundation (see file COPYING for details).

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
*/

/* $Id: getstats.c,v 1.1 2004/02/08 14:57:34 emes Exp $ */

#include "exit.h"
#include "getstats.h"
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <strings.h>

#define         BUFSIZE     512

double          stat_la1,
                stat_la5,
                stat_la15;
long int        stat_memfree,
                stat_memused,
                stat_swapfree,
                stat_swapused;
const char      *lafile = "/proc/loadavg";
int             lafha;
const char      *memfile = "/proc/meminfo";
int             memfha;
    
char            fbuf[ BUFSIZE ];


void fetch_la ()
{
    int         tmp;
    int         i;
    char        *one, *two, *three;

    lseek (lafha, 0, SEEK_SET);
    read (lafha, (char *) &fbuf, BUFSIZE);

    one   = &fbuf[0];
    two   = index (one, ' ') + 1;
    three = index (two, ' ') + 1;
    
    stat_la1  = atof (one);
    stat_la5  = atof (two);
    stat_la15 = atof (three);

#ifdef DEBUG
    printf ("fetch_la(): la1=%0.2f la5=%0.2f la15=%0.2f\n",
            stat_la1, stat_la5, stat_la15);
#endif
}

void fetch_mem ()
{
    char                *ptr;
    long int            vmtot, vmused, vmbuf, vmcache, vsused, vsfree;

    lseek (memfha, 0, SEEK_SET);
    read (memfha, (char *) &fbuf, BUFSIZE);

    /* scanning /proc/meminfo for needed values */

    /* 1. memory total is in second line after 'Mem:\ *' */
    ptr = fto_notspace (fto_space (fto_newline ((char *)&fbuf)));
    vmtot = atol (ptr);
    /* 2. used mem is next value */
    ptr = fto_notspace (fto_space (ptr));
    vmused = atol (ptr);
    /* 3. then there are: 'free' field, 'shared' field,
     *    and what we're looking for - buffers
     */
    ptr = fto_notspace (fto_space (
                fto_notspace (fto_space (
                        fto_notspace (fto_space (ptr))
                        ))
                ));
    vmbuf = atol (ptr);
    /* 4. cached is next field */
    ptr = fto_notspace (fto_space (ptr));
    vmcache = atol (ptr);

    /* 5. next line, swap values. first is 'Swap:', then total.
     *    next one is swap-used */
    ptr = fto_notspace (fto_space (
                fto_notspace (fto_space (
                        fto_newline (ptr)
                        ))
                ));
    stat_swapused = atol (ptr);
    /* 6. swap-free */
    ptr = fto_notspace (fto_space (ptr));
    stat_swapfree = atol (ptr);

    /* calculate real values, i.e. substract buffers and cache size */
    stat_memused = vmused - (vmbuf + vmcache);
    stat_memfree  = vmtot - stat_memused;

#ifdef DEBUG
    printf ("fetch_mem(): memfree=%d memused=%d swapfree=%d swapused=%d\n",
            stat_memfree, stat_memused, stat_swapfree, stat_swapused);
#endif
}

void open_files ()
{
    lafha = open (lafile, O_RDONLY);
    if (lafha < 0) bail_out (EXIT_IO, lafile);
    
    memfha = open (memfile, O_RDONLY);
    if (memfha < 0) bail_out (EXIT_IO, memfile);
}

/* fast text parsing helpers */

char * fto_newline (char *b)
{
    do {
        b++;
    } while ((*b != '\n') && 
             (*b != '\r') && 
             (*b != '\0'));
    /* skip to new line */
    if (*b != '\0') b++;
    return b;
}

char * fto_notspace (char *b)
{
    do {
        b++;
    } while ((*b == ' ') || 
             (*b == '\t') || 
             (*b == '\n') || 
             (*b == '\r'));
    return b;
}

char * fto_space (char *b)
{
    do {
        b++;
    } while ((*b != ' ') && 
             (*b != '\t') && 
             (*b != '\n') && 
             (*b != '\r') &&
             (*b != '\0'));
    return b;
}


/* $Id: getstats.c,v 1.1 2004/02/08 14:57:34 emes Exp $ */
